
#include "Arduino.h"
#define MAX_PWM_VALUE 400
// pin 7 = PC23 = PWML6
// pin 45  = PC18 = PWMH6
// pin 8 = PC22 = PWML5
// pin 44 = PC19 = PWMH5

#define NECHANT 128
#define SHIFT_ACCUM 25
uint32_t table_onde[NECHANT];
uint32_t accum1,accum2,increm;
volatile void (*TC0_function)();
char pwm_chan[2] = {6,5};
char pwm_pin_1[2] = {7,8};
char pwm_pin_2[2] = {45,44};
            
void declencher_timer(uint32_t ticks, volatile void (*function)()) {
   uint8_t clock = TC_CMR_TCCLKS_TIMER_CLOCK1; // horloge 84MHz/2=42 MHz
   uint32_t channel = 0;
   TC0_function = function;
   pmc_set_writeprotect(false);
   pmc_enable_periph_clk((uint32_t)TC0_IRQn);
   TC_Configure(TC0, channel, TC_CMR_WAVE | TC_CMR_WAVSEL_UP_RC | clock);
   TC0->TC_CHANNEL[channel].TC_RC = ticks;
   TC_Start(TC0, channel);
   TC0->TC_CHANNEL[channel].TC_IER=TC_IER_CPCS;
   TC0->TC_CHANNEL[channel].TC_IDR=~TC_IER_CPCS;
   NVIC_EnableIRQ(TC0_IRQn);
}
              
void TC0_Handler()
{
   TC_GetStatus(TC0, 0);
   (*TC0_function)();
}
              
void init_pwm(int i, uint32_t frequency, uint32_t ulValue) {
   pmc_enable_periph_clk(PWM_INTERFACE_ID);
   PWMC_ConfigureClocks(frequency*MAX_PWM_VALUE,0,VARIANT_MCK);
   uint32_t chan = pwm_chan[i];
   uint32_t ulPin1 = pwm_pin_1[i];
   uint32_t ulPin2 = pwm_pin_2[i];
   PIO_Configure(g_APinDescription[ulPin1].pPort,
					g_APinDescription[ulPin1].ulPinType,
					g_APinDescription[ulPin1].ulPin,
					g_APinDescription[ulPin1].ulPinConfiguration);
   PIO_Configure(g_APinDescription[ulPin2].pPort,
					g_APinDescription[ulPin1].ulPinType,
					g_APinDescription[ulPin2].ulPin,
					g_APinDescription[ulPin2].ulPinConfiguration);
   
  PWMC_ConfigureChannel(PWM_INTERFACE, chan, PWM_CMR_CPRE_CLKA, 0, 0);
  PWM_INTERFACE->PWM_CH_NUM[chan].PWM_CMR |= PWM_CMR_DTE; // dead time enable
  PWMC_SetPeriod(PWM_INTERFACE, chan, MAX_PWM_VALUE);
  PWMC_SetDutyCycle(PWM_INTERFACE, chan, ulValue);
  //uint32_t dead_time = MAX_PWM_VALUE/20;
  //PWMC_SetDeadTime(PWM_INTERFACE, chan, dead_time, dead_time);
  PWMC_EnableChannel(PWM_INTERFACE, chan);
}
                 
volatile void synthese_table() {
    accum1+= increm;
    accum2+= increm;
    PWM_INTERFACE->PWM_CH_NUM[pwm_chan[0]].PWM_CDTYUPD = table_onde[accum1 >> SHIFT_ACCUM];
    PWM_INTERFACE->PWM_CH_NUM[pwm_chan[1]].PWM_CDTYUPD = table_onde[accum2 >> SHIFT_ACCUM];
}
                  
void set_sinus_table(float amp) {
  int i;
  float dt = 2*3.1415926/NECHANT;
  for(i=0; i<NECHANT; i++) {
    table_onde[i] = MAX_PWM_VALUE*0.5*(1.0+amp*sin(i*dt));
  }  
}
                  
void setup() {
  set_sinus_table(1.0);
  uint32_t pwm_freq = 100000; // fréquence du PWM en Hz
  init_pwm(0,pwm_freq,0);
  init_pwm(1,pwm_freq,0);
  uint32_t frequence = 1000; // Fréquence de la sinusoïde en Hz
  uint32_t fechant = 42000; // Fréquence d'échantillonnage en Hz
  uint32_t techant = 1.0/fechant; // en secondes
  uint32_t ticks = 42000000/fechant; // nombre de tops d'horloges à 42 MHz pour la période d'échantillonnage
  increm = (uint32_t) (((float)(0xFFFFFFFF))*((float)(frequence)*techant)); // incrément de l'accumulateur de phase
  accum1 = 0;
  accum2 = ((uint32_t)(NECHANT * 0.25)) << SHIFT_ACCUM;
  declencher_timer(ticks,synthese_table);
}
                  
void loop() {
  

}
                  